#include <stdlib.h>
#include <uf_defs.h>
#include <uf_layout.h>
#include <uf_modl.h>
#include <uf_csys.h>
#include <uf_view.h>
#include <uf_obj.h>
#include <uf_drf.h>
#include "calcproto.h"

/**********************************************************************
*
* calc_dimension_button 
*
* Create dimensions on the button.  Use the routine that extracts edges
* to obtain the objects for dimensioning.  Edges are returned in a 
* specific order.
*     edge[0]   Button left front
*     edge[1]   Button right front
*     edge[2]   Button right rear
*
* revisions 
*   rv   date      who   reason for change 
*   00   01NOV95   J_P                   
*   01   01JUL96   J_P   Replaced uf5205 call with UF_OBJ_delete_object
*   02   01SEP97   J_P   Removed delete_object calls.  The set_wcs
*                        routine uses temporary coordinate systems.
*   03   14FEB03   DKF   Updated error checking
*
* input 
*  (none)  
*
* output 
*  calc_dimension_button       0 = success, >0 = error 
*  
*  
**********************************************************************/
    int calc_dimension_button()
{
/******************** Routine variable declaration  *******************/
    char
        radsym[7],      /* radius symbol string */
        diasym[7];      /* diameter symbol string */

    int mpi[100],       /* integer drafting parameters */
        epcode[3],      /* startpoint-or-endpoint flags*/
        flag;           /* results from functions */

    tag_t
        edge[3],            /* edges to be dimensioned */
        view_tag,           /* Tag of the view containing dimensions */
        dimension,          /* dimension object */
        csys1;              /* Original work coordinate system. */

    double
        mpr[70],        /* floating point drafting parameters */
        origin[3],      /* dimension origin */
        border,         /* border distance */
        button_height,  /* height of calculator */
        button_width;   /* width of calculator */

    UF_DRF_text_t
        dim_text;       /* Struct. for appended text and man. dim */

    UF_DRF_object_t
        dim0,           /* Structure for first object in dim. */
        dim1;           /* Structure for second object in dim. */

/******************** Beginning of Executable code *********************/


/*XXX Read the current settings for drafting/dimensioning */

/* Set the linear units to mm and change the character size and number
 * of decimal places.  XXX Call the routine that reset draft/dim settings.
 */
    mpi[3]  = 2;        /* decimal places */
    mpi[13] = 1;        /* linear units */
    mpr[32] = 5.0;      /* character height */

/*XXX Evaluate the expressions for the button height and width and
 * the border distance.  Check return codes.
 */

/*xxx Initialize structures.  This sets the associativity modifiers
 * to UF_DRF_end_point (which is what we want.  Check return codes!
 * Also initialize the text structure.
 */
    dim_text.lines_app_text = 0;
    dim_text.user_dim_text = NULL;
    dim_text.appended_text = NULL;

/* Get the edges for the dimensions.  They are returned in a particular
 * order documented in the comments at the routine start.
 */
    flag = calc_dimension_button_edges(edge,epcode);
    if(flag) return (flag);

/* Section to create horizontal dimensions */

/* Set the work view (and WCS) to create a dimension in the FRONT
 * view.  Save the original WCS in csys1.
 */
    UF_CALL( uc6464("","","FRONT") );
    UF_CALL( UF_CSYS_ask_wcs(&csys1) );
    flag = ug_set_wcs_to_view("FRONT");
    if(flag) return(flag);

/*XXX Obtain the tag of the work view.  It is used in the structures. */
    
    dim0.object_view_tag = dim1.object_view_tag = view_tag;

/* Set the object tags and assoc. modifiers.  XXX Then create a dimension
 * from the left edge to right edge.
 */
    origin[0] = border + 0.5*button_width;
    origin[1] = 12.0 + 2.0*mpr[32];
    origin[2] = 0.0;

/* Set the work view (and WCS) to create a dimension in the RIGHT
 *  view.
 */
    UF_CALL( uc6464("","","RIGHT") );
    flag = ug_set_wcs_to_view("RIGHT");
    if(flag) return(flag);

/*XXX Obtain the tag of the work view.  It is used in the structures. */
    
    dim0.object_view_tag = dim1.object_view_tag = view_tag;

/* Set the object tags and assoc. modifiers.  Then XXX create a dimension
 * from the front right edge to rear right edge.
 */
    origin[0] = border + 0.5*button_height;
    origin[1] = 12.0 + 2.0*mpr[32];
    origin[2] = 0.0;

/* Reset the layout to the top view. */
    UF_CALL( uc6464("","","TOP") );

/* Reset the WCS to the original csys.  */
    UF_CALL( UF_CSYS_set_wcs( csys1) );

/*XXX Fit the view.  Any temporary WCS's have been removed. */

    return(0);
}
