#include <stdlib.h>
#include <uf_defs.h>
#include <uf_modl.h>
#include <uf_drf.h>
#include <uf_layout.h>
#include <uf_view.h>
#include "calcproto.h"

/**********************************************************************
*
* calc_dimension_top 
*
* Create dimensions on the top half of the calculator.  Use the routine
* that extracts edges to obtain the objects for dimensioning.  Edges
* are:
*       For horizontal dimensions
*     edge[0]  Calc. left
*     edge[1]  Calc. right
*     edge[2]  Photocell left
*     edge[3]  Photocell right
*     edge[4]  Buttonhole left
*     edge[5]  Buttonhole right
*       For Vertical dimensions
*     edge[6]  Calc. bottom
*     edge[7]  Calc. top
*     edge[8]  LCD bottom
*     edge[9]  LCD top
*     edge[10]  Photocell bottom
*     edge[11]  Photocell top
*     edge[12]  Button bottom
*     edge[13]  Button top
*
* revisions 
*   rv   date      who   reason for change 
*   00   01NOV95   J_P                   
*   01   01JUL96   J_P   Changed to new API dimension calls
*   02   01SEP97   J_P   Changed to new API preferences calls
*   03   14FEB03   DKF   Added new header file to #include list
*
* input 
*  (none)  
*
* output 
*  calc_dimension_top       0 = success, >0 = error 
*  
*  
**********************************************************************/

    int calc_dimension_top()
{
/******************** Routine variable declaration  *******************/
    char
        radsym[7],          /* radius symbol string */
        diasym[7];          /* diameter symbol string */

    int mpi[100],           /* integer drafting parameters */
        epcode[14],         /* startpoint-or-endpoint modifier */
        flag;               /* results from functions */

    tag_t
        edge[14],           /* edges to be dimensioned */
        view_tag,           /* Tag of the view containing dimensions */
        dimension;          /* dimension object */

    double
        mpr[70],            /* floating point drafting parameters */
        origin[3],          /* dimension origin */
        height,             /* height of calculator */
        width;              /* width of calculator */

    UF_DRF_text_t
        dim_text;           /* Struct. for appended text and man. dim */

    UF_DRF_object_t
        dim0,               /* Structure for first object in dim. */
        dim1;               /* Structure for second object in dim. */

/******************** Beginning of Executable code *********************/


/*XXX Read the current settings for drafting/dimensioning */

/* Set the linear units to mm and change the character size and number
 * of decimal places.  
 *XXX Call the routine that reset draft/dim settings.
 */
    mpi[3]  = 2;        /* decimal places */
    mpi[13] = 1;        /* linear units */
    mpr[32] = 5.0;      /* character height */

/*XXX Evaluate the expressions for the calculator height and width.  Check
 * return codes.
 */

/*XXX Initialize the structures.  This sets the associativity modifiers
 * to UF_DRF_end_point (which is what we want.  Check return codes!
 * Also initialize the text structure.
 */
    dim_text.lines_app_text = 0;
    dim_text.user_dim_text = NULL;
    dim_text.appended_text = NULL;

/* Get the edges for the dimensions.  They are returned in a particular
 * order documented in the comments at the routine start.
 */
    flag = calc_dimension_top_edges(edge,epcode);
    if(flag) return (flag);

/*XXX Obtain the tag of the work view.  It is used in the structures. */
    
    dim0.object_view_tag = dim1.object_view_tag = view_tag;

/* First dim. is from left edge to right.  Set the location. */
    origin[0] = 0.5 * width;
    origin[1] = height + 6.0*mpr[32];
    origin[2] = 5.0;

/* XXX Create the dimension after setting the structure data for
 * both obect tags and assoc. modifiers.
 */

/*XXX Next dim is the left edge to photocell left edge. */
    origin[0] = 0.5 * width;
    origin[1] = height + 2.0*mpr[32];
    origin[2] = 5.0;


/*XXX Next dim is from the photocell left edge to photocell right edge */
    origin[0] = 0.5 * width;
    origin[1] = height + 4.0*mpr[32];
    origin[2] = 10.0;


/*XXX Next dim is left edge to buttonhole left edge. */
    origin[0] = 0.5 * width;
    origin[1] = -4.0*mpr[32];
    origin[2] = 10.0;


/*XXX Next dim. is the width of the buttonhole.  */
    origin[0] = 0.5 * width;
    origin[1] = -2.0*mpr[32];
    origin[2] = 10.0;


/* Create vertical dimensions.  */

/*XXX Create the dimension from the calculator bottom to top.  */
    origin[0] = width + 50.0;
    origin[1] = 0.5*height;
    origin[2] = 5.0;


/*XXX Next dim. is from the photocell top to calc. top.  */
    origin[0] = width + 15.0;
    origin[1] = height + 2.0*mpr[32];
    origin[2] = 5.0;


/*XXX Next dim. is the photocell height */
    origin[0] = width + 50.0;
    origin[1] = height + 4.0*mpr[32];
    origin[2] = 5.0;


/*XXX Next dim. is the distance from photocell bottom to LCD top.  */
    origin[0] = width + 15.0;
    origin[1] = 0.5*height;
    origin[2] = 5.0;


/*XXX Next dim. is the height of the LCD.  */
    origin[0] = width + 35.0;
    origin[1] = 0.5*height;
    origin[2] = 5.0;


/*XXX Next dim. is from the calc. bottom to the buttonhole bottom.  */
    origin[0] = -25.0;
    origin[1] = 0.5*height;
    origin[2] = 5.0;


/*XXX The final dimension is the height of the buttonhole.  */
    origin[0] = -60.0;
    origin[1] = 0.5*height;
    origin[2] = 5.0;


    return (0);
}
