#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <uf_defs.h>
#include <uf_obj.h>
#include <uf_modl.h>
#include <uf_assem.h>
#include <uf_ui.h>
#include "calcproto.h"

static int move_holes( uf_list_p_t, double, double, double, double );

/**********************************************************************
*
* calc_edit 
*
* Allow the user to change the overall length and with of the calculator.
* Properly update the assembly to reflect the changes.
*
* revisions 
*   rv   date      who   reason for change 
*   00   01NOV95   J_P                   
*
* input 
*    holes         List of photocell and lcd slots 
*
* output 
*    calc_edit     0 = success, > 0 = error 
*  
*  
**********************************************************************/

    int calc_edit(uf_list_p_t holes, tag_t top_inst,
                  tag_t bottom_inst, tag_t button_inst)
{
/******************** Routine variable declaration  *******************/
    char
        option[2][16],  /* option wording */
        hstr[132],      /* height expression */
        wstr[132];      /* width expression */

    int flag,           /* success flag from any operation */
        resp,           /* user response */
        i,              /* loop control */
        idum[2];

    tag_t
        wpart,          /* part associated with component */
        parent,         /* assembly */
        feat,           /* feature tag */
        body;           /* body of button geometry */

    double
        height,         /* height */
        width,          /* width  */
        ht0,            /* original height */
        wd0,            /* original width */
        rdata[3],       /* button area height and width */
        new_floc[3],    /* button feature location after move */
        old_floc[3];    /* button feature location before move */

    double
        bwidth,         /* button width   */
        bheight;        /* button height  */

    uf_list_p_t
        move_list;      /* List of features to move.  */

/******************** Beginning of Executable code *********************/


/*XXX Save the tag of the current work part (calculator) and XXX change
 * the work part to the top part. Check return codes/tags.
 */

/*XXX Evaluate the expressions for calculator height and width.  Set
 * the variables ht0 and wd0 with these original values.
 */

    ht0 = height;
    wd0 = width;

/* Loop to allow the user to key in new values for height/width.
 * Use the previous values for defaults.
 */
    strcpy(option[0],"Height");
    strcpy(option[1],"Width");
    resp = 3;

    while(resp > 2) {
        rdata[0] = height;
        rdata[1] = width;
/* Prompt user with the existing numbers.  */
        resp = uc1609(" Change vaues?", option, 2, rdata, idum );

/* Exit the loop if BACK or CANCEL are selected.  */
        if(resp <= 3) break;

/* Check for invalid input.  */
        if((rdata[0] < 25.0) || (rdata[1] < 25.0)) {
            uc1601("Minimum height and width values ae 25.0",1);
            continue;
        }

/*XXX Create the move list and XXX put the first two holes (the lcd
 * and photocell photocell slots) on that list.
 */
        if( flag ) return( flag );

        for( i=0; i<2; i++ ) {
        }

/* If the calculator is smaller, move the slots and update the height.
 * The calculator height must be changed first if it is larger.  Once
 * the calculator is larger, the new holes can be created.
 */
        if(rdata[0] < height) {

/* Move the hole features.  The old and new calculator size will be
 * used to determine the move distances.  Check the return code.
 */
            flag = move_holes( move_list, .5*width, height, 
                               .5*rdata[1], rdata[0] );
            if( flag ) {
                flag = UF_CALL(UF_MODL_delete_list( &move_list ));
                return( flag );
            }

/*XXX Delete the old move_list and XXX create a new list.
 * Check return codes.
 */

            sprintf(hstr,"Body_height=%f",rdata[0]);
            sprintf(wstr,"Body_width=%f",rdata[1]);
/*XXX Edit the height and width expressions.  Check return codes.  */

/*XXX Get the button hole feature from the hole list and XXX put it on
 * the  move list.  It is index 2 on the holes list. 
 * Check return codes.
 */

/* Set the the variable old_floc to the current button feature
 * location.  The 'feat' variable can be used in ask_feat_location.
 * Check return code.
 */
            flag = UF_CALL(UF_MODL_ask_feat_location( feat, old_floc ));
            if( flag ) {
                flag = UF_CALL(UF_MODL_delete_list( &move_list ));
                return( flag );
            }

/*XXX Evaluate the expressions for the Button_width and Button_height,
 * The 'new' button location is determined from these evaluations.
 */

            new_floc[0] = 5.0 + 0.5*bwidth;
            new_floc[1] = 5.0 + 0.5*bheight;
/* Move the button hole.  This will move the base feature of an instance
 * array and all holes will be moved.
 */
            flag = move_holes( move_list, old_floc[0], old_floc[1],
                   new_floc[0], new_floc[1] );
            if( flag ) {
                flag = UF_CALL(UF_MODL_delete_list( &move_list ));
                return( flag );
            }
        }
        else {
            sprintf(hstr,"Body_height=%f",rdata[0]);
            sprintf(wstr,"Body_width=%f",rdata[1]);
/*XXX Edit the height and width expressions.  Check return codes.  */

/*XXX Update the model to have the larger expressions take effect. */
            if( flag ) {
                flag = UF_CALL(UF_MODL_delete_list( &move_list ));
                return( flag );
            }

/* Move the lcd and photocell slot features using the new and old
 * height/width data.  Check return code.
 */
            flag = move_holes( move_list, .5*width, height,
                                 .5*rdata[1], rdata[0] );
            if( flag ) {
                flag = UF_CALL(UF_MODL_delete_list( &move_list ));
                return( flag );
            }

/*XXX Delete the old move list and XXX create a new list.
 * Check return codes.
 */

/*XXX Get the button hole feature from the hole list and XXX put it
 * on the move list.  It is index 2 on the holes list.
 * Check return codes.
 */

/* Set the the variable old_floc to the current button feature
 * location.  The 'feat' variable can be used in ask_feat_location.
 * Check return code.
 */
            flag = UF_CALL(UF_MODL_ask_feat_location( feat, old_floc ));
            if( flag ) {
                flag = UF_CALL(UF_MODL_delete_list( &move_list ));
                return( flag );
            }

/*XXX Evaluate the expressions for the Button_width and Button_height,
 * The 'new' button location is determined from these evaluations.
 */

            new_floc[0] = 5.0 + 0.5*bwidth;
            new_floc[1] = 5.0 + 0.5*bheight;
/* Move the button hole.  This will move the base feature of an instance
 * array and all holes will be moved.
 */
            flag = move_holes( move_list, old_floc[0], old_floc[1],
                   new_floc[0], new_floc[1] );
            if( flag ) {
                UF_CALL(UF_MODL_delete_list( &move_list ));
                return( flag );
            }
        }

/*XXX Delete the move list.  Check the return code.  */

/*XXX Update the model to reflect the change(s).  Check return code. */

/* Set the prompt values to the input values at the bottom of the 
 * loop.  They will be used at the top of the loop in uc1609.
 */
        height = rdata[0];
        width  = rdata[1];
    }


/* We have exited the loop prompting for new data.  If there have
 * been no changes, reset the work part to the root and return.
 */
    if((ht0 == height) && (wd0 == width)) {
        flag = UF_CALL(UF_PART_set_display_part(parent));
        return (flag);
    }

/*XXX If items have changed, export the expressions.  Delete the
 * current expression file before exporting.  Check return code.
 */
    system("rm -f ./calc.exp");

    if( flag ) return( flag );

/*XXX Set the work part to the bottom component by getting the child
 * part tag and XXX setting the work part to it.
 */

/*XXX Edit height/width expressions (hstr & wstr).
 *  Check return codes.
 */

/*XXX Update the bottom component.  Check return code! */


/*XXX Set the work part to the button component by XXX getting the
 * child part tag and setting the work part to it.
 */

/* Find the object named "BUTTON" and delete it.  We must re-create
 * the button object.  It has changed size but the slot feature has not
 * moved.  It will be easier to delete and recreate the button than to
 * reposition the slot.
 */
    body = NULL_TAG;
    flag = UF_CALL( UF_OBJ_cycle_by_name("BUTTON", &body) );
    if( flag ) return( flag );
    flag = UF_CALL(UF_OBJ_delete_object( body ) );
    if( flag ) return( flag );

/*XXX Edit height/width expressions (hstr & wstr).
 * Check return codes.
 */

/*XXX Update the model.  Check return code! */

/*XXX Reset the color for the button. */

/* Re-create the button.  It will use the new height/width.  */
    flag = calc_model_button();
    if(flag) return (flag);

/*XXX Set the display part to the calculator 
 * (the 'parent' part tag).
 */

/*XXX Edit height/width expressions (hstr & wstr).
 * Check return codes.
 */

/*XXX Update the model.  Check return code! */

/* Add the dimensions to the newly created button. */
    flag = calc_dimension_button();
    return (flag);
}



static int move_holes( uf_list_p_t feats, double oldx, double oldy,
                    double newx, double newy )
{
    int flag;       /* success flag from any operation */

    double
        mat[2][12]; /* reference and destination matrices */


/* Set up the translation matrix to move the photocell and slot features
 * based on the new input.
 */
    mat[0][3] = mat[1][3] = 1.0;
    mat[0][4] = mat[1][4] = 0.0;
    mat[0][5] = mat[1][5] = 0.0;
    mat[0][6] = mat[1][6] = 0.0;
    mat[0][7] = mat[1][7] = 1.0;
    mat[0][8] = mat[1][8] = 0.0;
    mat[0][9] = mat[1][9] = 0.0;
    mat[0][10] = mat[1][10] = 0.0;
    mat[0][11] = mat[1][11] = 1.0;
    mat[0][0] = oldx;
    mat[0][1] = oldy;
    mat[0][2] = 0.0;
    mat[1][0] = newx;
    mat[1][1] = newy;
    mat[1][2] = 0.0;

/* XXX Move the list of features passed into the routine using the
 * matrix established by the translational values passed into the
 * routine.
 */
    return( flag );
}
