#include <string.h>
#include <math.h>
#include <uf_defs.h>
#include <uf_modl.h>
#include <uf_obj.h>
#include <uf_attr.h>
#include <uf_part.h>
#include "calcproto.h"

/**********************************************************************
* calc_model_top
*  
* create the top half of the calculator 
*
* revisions 
* rv   date      who   reason for change 
* 00   01NOV95   J_P                   
* 01   01AUG97   J_P   changed some variable names and limited scope of
*                      other variables.
* 02   01JUN98   J_P   initialized some variables
* 03   14FEB03   DKF   Removed unreferenced local variables
* 04   14FEB03   DKF   Updated part attribute assignment procedure
*  
* input 
*  (n1) 
*  
* output 
*  holes                   List of photocell and lcd slots 
*  calc_model_top          0 = success, >0 = error 
*
**********************************************************************/

    int calc_model_top(uf_list_p_t holes)
{
/******************** Routine variable declaration  *******************/
    char
        *edge_len_p[3],     /* Pointers to block edge length strings. */
        edge_len[3][133],   /* Strings for edge length expressions. */
        svalue[50];         /* string for attribute value */

    int flag=0,             /* 0 = success, >0 = error */
        ecount=0,           /* count of objects in list */
        i=0;                /* looper */

    int allow_smooth=0,     /* allow all blends */
        allow_cliff=0,
        allow_notch=0;

    tag_t
        part,               /* part tag, for attribute assignment */
        top_half,           /* top half feature */
        body,               /* top_half body    */
        hollow,             /* hollow feature */
        blend1,blend2;      /* blend features */

    uf_list_p_t
        list1,              /* list of edges */
        list2;              /* list for storing tags for form feats. */

    double
        corner[3],          /* start corner of block */
        pt1[3],             /* coordinates of vertex */
        pt2[3];             /* coordinates of vertex */

    double
        vrb_tol=0.001;      /* variable radius blend tolerance */

    UF_ATTR_value_t
        att_value;          /* attribute data structure */

/******************** Beginning of Executable code *********************/


/* Set up data for block creation.  The routine takes an array of
 * pointers to strings.  Set edge_len_p to the addresses of the 
 * edge_len array items.
 */
    corner[0] = 0.0;
    corner[1] = 0.0;
    corner[2] = 5.0;

    edge_len_p[0] = &edge_len[0][0];
    edge_len_p[1] = &edge_len[1][0];
    edge_len_p[2] = &edge_len[2][0];

    strcpy(edge_len[0],"Body_width");
    strcpy(edge_len[1],"Body_height");
    strcpy(edge_len[2],"5.0");

/*XXX Create the block and check the return code */

/*XXX Obtain the body tag from the feature tag. XXX Assign the name
 * TOP_HALF to the body.  Check the return code.
 */

/*YYY  Attribute code will be added after the model code is completed */
/* Add part number and description attributes to the part.  Obtain the 
 * display part tag. .
 * For UF_ATTR_assign, use the titles
 * "PARTNUMBER" and "DESCRIPTN"
 * and the values shown below.
 */
/*YYY Obtain the part tag for the current part */

    att_value.type = UF_ATTR_string;
    strcpy(svalue,"001");   /* First call is the part number. */
    att_value.value.string = svalue;
    /*YYY Assign the attribute */

    strcpy(svalue,"TOP");   /* Second call for description. */
    /*YYY Assign the attribute */


/*XXX Get the edges of the body. XXX Get the count of the edge list.
 * This will be used to get the four 'vertical' corners of the block for
 * blending. Check return codes.
 */


/*XXX Create list2,  a list of objects.  The edges to blend will populate
 * this list.
 */

/* Loop through all object edges and get their vertices.  Use the edge
 * vertices to determine which edges to blend.  The edges we want have
 * a Z coordinate change of 5 mm.  Place the id's of the edges to blend
 * in a second list (list2).
 */

    for(i=0; i < ecount; i++) {
        tag_t
            edge;               /* edge object */

        int vcount;             /* count of vertices in edge */

        /*XXX Get the edge (list item) and check return code.  */

        /*XXX Get the edge vertices.  Check return code.  */

        if(fabs(fabs(pt1[2] - pt2[2]) - 5.0) < 0.001) {
            /*XXX Put item on list2.  Check return code.  */
        }
    }

/*XXX Blend the qualified edges with a 5 mm radius.  Check return code.  */

/*XXX Delete the previous edge lists.  Since the block has been edited
 * (blended), the first edge list may no longer be valid.
 */

/*XXX Get a new list of body edges.  XXX Create a second list (list2)
 * for qualifying edges.  XXX Get the count of edges (ecount) in the
 * body edge list.  Check return codes.
 */

/* Loop through new object edges and get their vertices.  Use the edge
 * vertices to determine which edges are on the top of the body.  Edges
 * with Z coordinates of both vertices at 10 mm should populate list2.
 */

    for(i=0; i < ecount; i++) {
        tag_t
            edge;               /* edge object */

        int vcount;             /* count of vertices in edge */

        /*XXX Get the edge (list item) and check return code.  */

        /*XXX Get the edge vertices.  Check return code.  */

        if((fabs(pt1[2] - 10.0) < 0.001)
            && (fabs(pt2[2] - 10.0) < 0.001)) {  
            /*XXX Put item on list2.  Check return code.  */
        }
    }

/*XXX Blend the qualified edges with a 2 mm radius.  Check return code. */

/*XXX Delete the lists used for the previous blend operation. */

/*XXX Create list2 to store the face used in the hollow operation.
 *XXX Obtain the faces of the body.  Check return codes! 
 */
 
/*XXX Obtain the count of faces (ecount) and loop through the face
 * objects.  We want the face with a normal in the -Z direction.
 */

    for(i=0; i < ecount; i++) {
        tag_t
            face;               /* face object      */

        int ftype,              /* face type */
            dsense;             /* face direction sense */

        double
            nrm[3],             /* face normal direction vector */
            box[6],             /* face boundary         */
            rad1,               /* face major radius     */
            rad2;               /* face minor radius     */

        /*XXX Get a face (list item) from the list.  */

        /*XXX Use UF_MODL_ask_face_data to get the normal direction.  */

        if(fabs(nrm[2] + 1.0) < 0.001) {
            /*XXX Got the face!  Put it on the list and break out of the loop. */
            break;
        }
    }

/*XXX Perform the hollow operation.  Use a 1mm thickness.
 * Check return code.
 */

/*XXX Delete the lists used to set up and perform the hollow. */

/* Create the holes (pockets) */
/*    flag = calc_model_top_holes(body,holes); */
    return (flag);
}
