/*===========================================================================

                  Copyright (c) 1998 Electronic Data Systems Corporation
                             Unpublished - All rights reserved

 ===========================================================================*/
#include <stdio.h>
#include <string.h>
#include <uf_defs.h>
#include <uf.h>
#include <uf_ui.h>
#include <uf_part.h>

/******************************************************************************/
/*         Define variables used in this set of routines.                     */

#define  WAIT      ((int) 1)    /* For uc1601 to wait for OK after msg.       */
#define  NOWAIT    ((int) 0)    /* For uc1601 to display in status line.      */
/******************************************************************************/

/******************************************************************************/
/*      Protoype internal routines.  Make then static, if possible.           */

static void session_stats( void );

/******************************************************************************/

/*******************************************************************************
*   Lead routine.  Initializes Ufunc (checks out a license) and calls the      *
*   session statistics routine.  Once the stats routine returns, it terminates *
*   (checks back in a license).                                                *
*******************************************************************************/
extern void ufusr ( char *param,
                    int *retcod,
                    int parmlen )
{
/************************ Routine variable declaration  ***********************/
    int irc;        /*  The return code from called routines  */


/************************ Beginning of Executable code *************************/

    irc = UF_initialize();
    if( irc ) {
        char
            messg[MAX_LINE_SIZE + 1];

        UF_get_fail_message( irc, messg );
        printf(" Unable to initialize Ufunc program.  Error was:\n%s\n", messg);
    }
    else {
        session_stats();
        irc = UF_terminate();
    }
}

/*******************************************************************************
*   This routine sets up the listing device for output to the CRT.  It         *
* displays the Ufunc mode (internal/external), the UG revision, and part       *
* name/units (if there are parts open).                                        *
*******************************************************************************/
static void session_stats( )
{
/************************ Routine variable declaration  ***********************/
    int
        irc,            /* The return code from called routines */
        mode,           /* 0 = external, 1 = internal;  used as an index
                           for chrmod */
        num_prts,       /* number of open part files */
        i;              /* Loop counter */

    logical
        open;           /* Flag for open/close status of listing window */
    char
        *chrmod[]={ "External", "Internal" },  /* The type of Ufunc program */
        crtline[MAX_LINE_SIZE + 1],            /* For uc4403 ( and sprintf) */
        *release;                              /* Pointer to string with the
                                                 release.  Must be freed!!  */

    tag_t
        dptag;           /* The tag of the displayed part (NULL_TAG if none) */

/************************ Begining of Executable code *************************/

/* Open the listing window if it is not already open. */
    irc = UF_UI_is_listing_window_open( &open );
    if( !open ) {
        irc = UF_UI_open_listing_window( );
        if( irc ) {
            uc1601("Unable to open listing device.  Exiting", WAIT );
            return;
        }
    }


/* Get the UG release and print it to the listing device */
    irc = UF_get_release( &release );
    if( irc ) {
        uc1601("Unable to obtain UG release.  Continuing", WAIT );
    }
    sprintf( crtline, " *****Using Unigraphics release %s *****\n", release );
/* Only free release if there was no error from UF_get_release */
    if( !irc )
        UF_free( release );
    irc = UF_UI_write_listing_window( crtline );
    if( irc ) {
        uc1601("Unable to write release to listing device.  Continuing", WAIT );
    }

/* Check for parts.  If there are open parts, cycle the parts and obtain the
 * units.  Demark the display part by adding a '(display)' after the name.
 * Note that part numbers run from 0 to num_prts-1.
 */

    num_prts = UF_PART_ask_num_parts( );
    dptag = UF_PART_ask_display_part ();

    for( i=0; i < num_prts; i++ ) {
        int
            punit;    /* Part units.  Use punit-1 as index to chrpun
                           character array (to convert number to string) */
        tag_t
            ptag;    /* 'Current' part tag. */
        char
            *chrpun[]={ "Metric", "English" },    /* The part units. */
            pname[MAX_LINE_SIZE + 1];            /* Part file name  */

        sprintf( crtline, "  \n" );        /*  Blank line */
        irc = UF_UI_write_listing_window( crtline );
/* Obtain tag, name, and units of the part. */
        ptag = UF_PART_ask_nth_part( i );
        UF_PART_ask_part_name( ptag, pname );
		/*
        punit = UF_PART_ask_part_units( ptag );
		This call was changed in V14 to return an errorcode and use the
		provided integer space to store the units code.
		*/
        irc = UF_PART_ask_units( ptag, &punit );

/* Check if the current part is the display part;  append text if this is true.
 */
        sprintf( crtline, "%s", pname );
        if( ptag == dptag )
            strcat( crtline, "  (*****DISPLAY*****)");
        strcat(crtline, "\n" );
        irc = UF_UI_write_listing_window( crtline );
        if( irc ) {
            uc1601("Unable to write part file name.  Continuing", NOWAIT );
        }

/* Recall that the part unit character array was set up to handle the output
 * (decremented by 1 - *** changed in V14 ***) of the ask_part_unit routine.
 */
        sprintf( crtline, "has %s units and is open in the current session.\n",
            /* chrpun[ punit ] );

			Changed for V14 */
            chrpun[ punit-1 ] );

        irc = UF_UI_write_listing_window( crtline );
        if( irc ) {
            uc1601("Unable to write part file units.  Continuing", NOWAIT );
        }
    }
}
