/*===========================================================================
 
                  Copyright (c) 1997 Electronic Data Systems Corporation
                             Unpublished - All rights reserved
 
 ===========================================================================*/
#include <stdio.h>
#include <string.h>
#include <uf.h>
#include <uf_defs.h>
#include <uf_disp.h>
#include <uf_ui.h>
#include <uf_object_types.h>

#define    UF_CALL(X) (report( #X, __FILE__, __LINE__, (X) ))

static int report( char *call, char *file, int line, int irc )
{
    if( irc ) {
        char messg[133];
        printf("%s\n%s, line %d:  ", call, file, line );
        (UF_get_fail_message( irc, messg )) ?
            printf("Returned %d\n", irc ):
            printf("Error %d:  %s\n", irc, messg );
    }
    return( irc );
}

/******************************************************************************/
/*    Protoype internal routines.  Make then static, if possible.             */

static void select_geo( tag_p_t *, int * );
/******************************************************************************/

/*******************************************************************************
*    Lead routine.  Initializes Ufunc (checks out a license) and calls the      *
*    geometry selection routine.  Once the geometry is selected, the array is   *
*    freed and the Ufunc is terminated (checks back in a license).              *
*******************************************************************************/
void ufusr ( char *param,
             int *retcod,
             int parmlen )
{
/********************  Routine variable declaration  **************************/
    int irc;            /* The return code from called routines */

/********************  Begining of executable code  ***************************/

    
    if( !UF_CALL(UF_initialize()) ) {
        int count,         /* The number of objects picked. */
             i;            /* Loop counter */
        tag_t
             *picks;        /* Pointer to an array of (UG allocated) objects */

        select_geo( &picks, &count);
        for( i=0, printf(" No. objects selected = %d\n",count); i<count; i++ ) {
            printf(" Object %d was id %d\n", i+1, picks[i] );
        }
        if( count )
            UF_free( picks );

        UF_CALL(UF_terminate());
    }
}

/*******************************************************************************
*   This routine allows picking of faces and edges.  The array of selected     *
* faces/edges and the number of objects selected is returned.  The routine     *
* returns a count of zero and a NULL tag if an error occurs.  Selection is     *
* allowed for any objects in an assembly (occurrences or prototypes).          *
*    Selected objects are unhighlighted.                                       *
*******************************************************************************/
static void select_geo( tag_p_t *picks,
                        int *count )
{
/********************  Routine variable declaration  **************************/
    int
        response,           /*    Return value from selection routine */
        irc;                /* Return code from selection routine */

    char
        *title="Select faces and edges";    /* String for prompt in cue line.*/

    UF_UI_selection_options_t
        selex;    /* structure for selection routine info (scope, mask) */

    UF_UI_mask_t
        mask[2];    /* array of mask information (triples) describing the
                       types of entities being selected.  */


/********************  Begining of executable code  ***************************/
    
/* Initialize returned variables. */
    *count = 0;
    *picks = NULL;

/* Set up the mask for faces and edges.  Tell selection routine that
 * two 'mask triples' (face and edge) are being passed.  Set the 
 * pointer of the selex mask_triples member to the mask structure
 * array.  Since mask[2] is an array of structures, 'mask' is the
 * address (pointer) to those structures.
 */
    selex.scope = UF_UI_SEL_SCOPE_ANY_IN_ASSEMBLY;
    selex.num_mask_triples = 2;
    selex.mask_triples = mask;

    mask[0].object_type = UF_solid_type;
    mask[0].object_subtype = 0;
    mask[0].solid_type = UF_UI_SEL_FEATURE_ANY_EDGE;
    mask[1].object_type = UF_solid_type;
    mask[1].object_subtype = 0;
    mask[1].solid_type = UF_UI_SEL_FEATURE_ANY_FACE;

    irc = UF_CALL(UF_UI_select_by_class( title, &selex, &response, count,
                  picks));
    if( irc ) {
            /* Set returned variables so garabge is not returned. */
        *count = 0;
        *picks = NULL;
    }
    else {
        int        unhighlight=0, i;

        for( i=0; i<(*count); i++ ) {
            UF_DISP_set_highlight ( (*picks)[i], unhighlight );
        }
    }
}
