#include <string.h>
#include <math.h>
#include <uf.h>
#include <uf_defs.h>
#include <uf_obj.h>
#include <uf_assem.h>
#include <uf_object_types.h>
#include <uf_modl.h>
#include "calcproto.h"

/**********************************************************************
* calc_comp_array 
*  This routine makes the rectangular array of components by setting
*  up the structure for a master component instance array.
*
* input 
*    button_inst The button component instance tag.
*
* output 
*    calc_comp_array    0 = success, else failure 
*
*   revisions 
*   rv     date    who     reason for change 
*   00   01JUL96   J_P                   
*   01   01AUG97   J_P     Added a call to free the face list
*   02   01JUN98   J_P     Initialized some variables
*   03   14FEB03   DKF     Removed unreferenced variables,
*                          Updated error checking
*  
*  
**********************************************************************/

      int calc_comp_array( tag_t button_inst )
{
/******************** Routine variable declaration  *******************/
    int i,          /* Loop variable */
        count=0;    /* Count of faces. */

    tag_t
        body,       /* Tag of the TOP_HALF body. */
        button_body,/* Tag of the original BUTTON body. */
        array,      /* Tag of the component instance array */
        *occ=NULL;  /* Ptr to occurrence tag of the face */

    uf_list_p_t
        list=NULL;  /* Face list from body. */

    char
        inst_array_name[MAX_ENTITY_NAME_SIZE+1];

    UF_ASSEM_mc_array_data_t
        linear_mc;  /* Master component structure. */

/******************** Beginning of Executable code *********************/

/* get the tag of the original "BUTTON" body */
    button_body = NULL_TAG;
    UF_CALL(UF_OBJ_cycle_by_name("BUTTON", &button_body));

/* Initialize the structure to create the component array of Buttons.
 * It will be a master component linear instance array using the same
 * expressions (Up, Across, etc.) used to create the hole instances.
 */

    linear_mc.array_subtype = UF_ASSEM_linear_array;
    linear_mc.master_component = NULL_TAG;
    linear_mc.template_component = button_inst;
    strcpy( inst_array_name, "BUTTON_ARRAY");
    linear_mc.array_name = inst_array_name;

/* Set the linear_mc.dimensions[0..1] values to the tags of the
 * Across and Up expressions.
 */
    UF_CALL(UF_MODL_create_exp_tag( "Across", &(linear_mc.dimensions[0]) ));
    UF_CALL(UF_MODL_create_exp_tag( "Up", &(linear_mc.dimensions[1]) ));

/* Set the linear_mc.offsets[0..1] values to the tags of expressions
 * Button_width+Button_spacing  and  Button_height+Button_spacing
 */
    UF_CALL(UF_MODL_create_exp_tag( "Button_width+Button_spacing", &(linear_mc.offsets[0]) ));
    UF_CALL(UF_MODL_create_exp_tag( "Button_height+Button_spacing", &(linear_mc.offsets[1]) ));

/* Get the body with the name "TOP_HALF".  Get a list of the
 * faces.  Get the list count.  The face normals will be checked
 * for normals in the global X and global Y (to define the direction
 * of the component array).
 */

    body = NULL_TAG;
    UF_CALL(UF_OBJ_cycle_by_name("TOP_HALF", &body));
    UF_CALL(UF_MODL_ask_body_faces( body, &list ));
    UF_CALL(UF_MODL_ask_list_count( list, &count ));

    for( i=0; i<count; i++ ) {
        tag_t face;
        int type, norm_dir;
        double point[3], nrm[3], box[6], radius, rad_data;

/* Obtain the face from the list */
        UF_CALL(UF_MODL_ask_list_item( list, i, &face ));

/* Obtain the face data for the normal information */
        UF_CALL(UF_MODL_ask_face_data( face, &type, point,
                nrm, box, &radius, &rad_data, &norm_dir));
        if( type != UF_bounded_plane_subtype )
            continue;
        if(fabs(nrm[0] - 1.0) < 0.00001)
            linear_mc.axis_definitions[0] = face;
        if(fabs(nrm[1] - 1.0) < 0.00001)
            linear_mc.axis_definitions[1] = face;
    }

/* Free the list of faces.  Check return code.  */
    UF_CALL(UF_MODL_delete_list( &list ) );

/*  Create the master component array.  Check return codes.  */
    UF_CALL(UF_ASSEM_create_mc_array( &linear_mc, &array ) );

/* Rename the original "BUTTON" body to "BUTTONONE" for use later */
    UF_CALL( UF_OBJ_set_name(button_body,"BUTTONONE") );

    return (0);
}

