#include <math.h>
#include <string.h>
#include <uf.h>
#include <uf_defs.h>
#include <uf_modl.h>
#include <uf_obj.h>
#include <uf_drf.h>
#include <uf_assem.h>
#include "calcproto.h"

/**********************************************************************
*
* calc_dimension_top_edges 
*
* get the edges to be dimensioned for the top half 
*
* revisions 
*   rv   date      who   reason for change 
*   00   01NOV95   J_P                   
*   01   01JUL96   J_P   Used V11 #defines for first/last point
*   02   01AUG97   J_P   Changed some obsolete calls to new API calls.
*   03   14FEB03   DKF   Removed unreferenced variable,
*                        Updated error checking
*
* input 
*  (none)  
*
* output 
* edge                            array of edges 
* epcode                          array of edge codes 
*                                    1 = dimension to start point 
*                                    2 = dimension to end point 
*  calc_dimension_top_edges       0 = success, >0 = error 
*  
*  
**********************************************************************/

    int calc_dimension_top_edges(tag_t *edge, int *epcode)
{
/******************** Routine variable declaration  *******************/
    int ecount,         /* count of edges */
        vcount,         /* count of vertices */
        i;              /* loop variable */

    tag_t
        body,           /* solid body */
        edge_tag;       /* any edge */

    uf_list_p_t
        elist;          /* list of edges */

    double
        pt1[3],         /* start point of edge */
        pt2[3],         /* end point of edge */
        height,         /* height of calculator */
        width,          /* width of calculator */
        lcd_height,     /* height of lcd window */
        lcd_pc_height,  /* space between lcd and pc windows */
        pc_height,      /* height of photocell window */
        pc_width,       /* width of lcd and pc windows */
        border,         /* height or width of button border */
        button_height,  /* button height */
        button_width;   /* button width */

/******************** Beginning of Executable code *********************/


/* Evaluate expressions for calculator */
    UF_CALL( UF_MODL_eval_exp("Body_height",&height ));
    UF_CALL( UF_MODL_eval_exp("Body_width",&width) );
    UF_CALL( UF_MODL_eval_exp("lcd_height",&lcd_height) );
    UF_CALL( UF_MODL_eval_exp("lcd_pc_height",&lcd_pc_height) );
    UF_CALL( UF_MODL_eval_exp("pc_height",&pc_height) );
    UF_CALL( UF_MODL_eval_exp("pc_width",&pc_width) );
    UF_CALL( UF_MODL_eval_exp("Border",&border) );
    UF_CALL( UF_MODL_eval_exp("Button_height",&button_height) );
    UF_CALL( UF_MODL_eval_exp("Button_width",&button_width) );

/* Locate the target body using it's name;  TOP_HALF  */
    body = NULL_TAG;
    UF_CALL( UF_OBJ_cycle_by_name("TOP_HALF",&body) );

/* Obtain the edges and count of edges in the body.  */
    UF_CALL( UF_MODL_ask_body_edges(body,&elist) );
    UF_CALL( UF_MODL_ask_list_count(elist,&ecount) );

/* Loop through the edges and find the vertices.  Check for specific
 * edges and store those particular edges and the appropriate end 
 * point code.  A code of 1 indicates the first vertex is the desired
 * endpoint;  code 2 is for the second endpoint.
 */
    for(i = 0; i < ecount; i++) {
        UF_CALL( UF_MODL_ask_list_item(elist,i,&edge_tag) );
        UF_CALL( UF_MODL_ask_edge_verts(edge_tag,pt1,pt2,&vcount) );

/* Check for vertical edges.  */
        if((fabs(pt1[0] - pt2[0]) < 0.001)) {

/* Look for left vertical edge at the back */
            if((fabs(pt1[0]) < 0.001) && (fabs(pt1[2] - 5.0) < 0.001)) {
                edge[0] = edge_tag;
                if(pt1[1] > pt2[1])
                    epcode[0] = UF_DRF_first_end_point;
                else
                    epcode[0] = UF_DRF_last_end_point;
                continue;
            }

/* Look for right vertical edge at back */
            if((fabs(pt1[0] - width) < 0.001)
              && (fabs(pt1[2] - 5.0) < 0.001)) {
                edge[1] = edge_tag;
                if(pt1[1] > pt2[1])
                    epcode[1] = UF_DRF_first_end_point;
                else
                    epcode[1] = UF_DRF_last_end_point;
                continue;
            }

/* Look for left edge of pc window */
            if((fabs(pt1[0] - border) < 0.001)
              && (fabs(pt1[2] - 10.0) < 0.001)
              && (fabs(fabs(pt1[1] - pt2[1]) - pc_height) < 0.001)) {
                edge[2] = edge_tag;
                if(pt1[1] > pt2[1])
                    epcode[2] = UF_DRF_first_end_point;
                else
                    epcode[2] = UF_DRF_last_end_point;
                continue;
            }

/* Look for right edge of pc window */
            if((fabs(pt1[0] - (border + pc_width)) < 0.001)
              && (fabs(pt1[2] - 10.0) < 0.001)
              && (fabs(fabs(pt1[1] - pt2[1]) - pc_height) < 0.001)) {
                edge[3] = edge_tag;
                if(pt1[1] > pt2[1])
                    epcode[3] = UF_DRF_first_end_point;
                else
                    epcode[3] = UF_DRF_last_end_point;
                continue;
            }

/* Look for left edge of button hole */
            if((fabs(pt1[0] - 4.75 ) < 0.001)
              && (fabs(pt1[2] - 10.0) < 0.001)) {
                edge[4] = edge_tag;
                if(pt1[1] > pt2[1])
                    epcode[4] = UF_DRF_first_end_point;
                else
                    epcode[4] = UF_DRF_last_end_point;
                continue;
            }

/* Look for right edge of hole border */
            if((fabs(pt1[0] - (border + button_width + 0.25)) < 0.001)
              && (fabs(pt1[2] - 10.0) < 0.001)) {
                edge[5] = edge_tag;
                if(pt1[1] > pt2[1])
                    epcode[5] = UF_DRF_first_end_point;
                else
                    epcode[5] = UF_DRF_last_end_point;
                continue;
            }
        }
 /* Check if edge is horizontal */
        else if(fabs(pt1[1] - pt2[1]) < 0.001) {

/* look for top edges on back */
            if((fabs(pt1[1]) < 0.001) && (fabs(pt1[2] - 5.0) < 0.001)) {
                edge[6] = edge_tag;
                if(pt1[0] > pt2[0])
                    epcode[6] = UF_DRF_first_end_point;
                else
                    epcode[6] = UF_DRF_last_end_point;
                continue;
            }

/* look for bottom edge on back */
            if((fabs(pt1[1] - height) < 0.001)
              && (fabs(pt1[2] - 5.0) < 0.001)) {
                edge[7] = edge_tag;
                if(pt1[0] > pt2[0])
                    epcode[7] = UF_DRF_first_end_point;
                else
                    epcode[7] = UF_DRF_last_end_point;
                continue;
            }

/* look for bottom edge of lcd window */
            if((fabs(pt1[1] - (height - border - pc_height -
              lcd_pc_height - lcd_height)) < 0.001)
              && (fabs(pt1[2] - 10.0) < 0.001)) {
                edge[8] = edge_tag;
                if(pt1[0] > pt2[0])
                    epcode[8] = UF_DRF_first_end_point;
                else
                    epcode[8] = UF_DRF_last_end_point;
                continue;
            }

/* look for top edge of lcd window */
            if((fabs(pt1[1] - (height - border - pc_height -
              lcd_pc_height)) < 0.001)
              && (fabs(pt1[2] - 10.0) < 0.001)) {
                edge[9] = edge_tag;
                if(pt1[0] > pt2[0])
                    epcode[9] = UF_DRF_first_end_point;
                else
                    epcode[9] = UF_DRF_last_end_point;
                continue;
            }

/* look for bottom edge of pc window */
            if((fabs(pt1[1] - (height-border-pc_height)) < 0.001)
              && (fabs(pt1[2] - 10.0) < 0.001)) {
                edge[10] = edge_tag;
                if(pt1[0] > pt2[0])
                    epcode[10] = UF_DRF_first_end_point;
                else
                    epcode[10] = UF_DRF_last_end_point;
                continue;
            }

/* look for top edge of pc window */
            if((fabs(pt1[1] - (height-border)) < 0.001)
              && (fabs(pt1[2] - 10.0) < 0.001)) {
                edge[11] = edge_tag;
                if(pt1[0] > pt2[0])
                    epcode[11] = UF_DRF_first_end_point;
                else
                    epcode[11] = UF_DRF_last_end_point;
                continue;
            }

/* look for bottom edge of buttonhole */
            if((fabs(pt1[1] - 4.75 ) < 0.001)
              && (fabs(pt1[2] - 10.0) < 0.001)) {
                edge[12] = edge_tag;
                if(pt1[0] > pt2[0])
                    epcode[12] = UF_DRF_first_end_point;
                else
                    epcode[12] = UF_DRF_last_end_point;
                continue;
            }

/* look for top edge of buttonhole */
            if((fabs(pt1[1] - (border+button_height+0.25)) < 0.001)
              && (fabs(pt1[2] - 10.0) < 0.001)) {
               edge[13] = edge_tag;
                if(pt1[0] > pt2[0])
                    epcode[13] = UF_DRF_first_end_point;
                else
                    epcode[13] = UF_DRF_last_end_point;
               continue;
            }
        }           /* endif */
    }              /* endloop */

/* Delete the list of edges before returning.  */
    UF_CALL( UF_MODL_delete_list(&elist) );

    return (0);
}
