#include <math.h>
#include <uf_defs.h>
#include <uf_modl.h>
#include <uf_view.h>
#include <uf_draw.h>
#include "calcproto.h"

/**********************************************************************
*
* calc_drawing_views 
*
*  Place the custom views FRONT_DRF, RIGHT_DRF, TOP_DRF and TFR-ISO_DRF 
*  views on the drawing.
*
* revisions 
*   rv   date      who   reason for change 
*   00   01NOV95   J_P                  
*   01   01JUL96   J_P   Added calls to set view drawing ref pt.
*   02   14FEB03   DKF   Minor corrections,
*                        Updated error checking,
*                        removed unreferenced local variable
*
* input 
*    dname                 drawing name 
*    dsize                 drawing size 
*
* output 
* calc_drawing_views        0=success, else failure 
*  
*  
**********************************************************************/

    int calc_drawing_views(char *dname,int dsize)
{
/******************** Routine variable declaration  *******************/

    tag_t
        view_tag;       /* Tag of TOP@ view for border resizing */

    double
        refpt[2],       /* drawing reference point */
        height,         /* calculator height       */
        width,          /* calculator width        */
        hspace,         /* horizontal space between views */
        vspace,         /* vertical space between views */
        center[3],      /* new view center */
		fraction=1.0;   /* fit per cent for UF_VIEW_fit_view */

    double
        htwd[][2] =  { 841.0, 1189.0,
                       594.0,  841.0,
                       420.0,  594.0,
                       297.0,  420.0,
                       210.0,  297.0 };

/******************** Beginning of Executable code *********************/


/* Evaluate the expressions for calculator height and width.  Check
 * return codes.
 */
    UF_CALL( UF_MODL_eval_exp("Body_height", &height ) );
    UF_CALL( UF_MODL_eval_exp("Body_width", &width ) );

/* Calculate spacing between views */
    hspace = (htwd[dsize-1][1] - width - height)/3.0;
    vspace = (htwd[dsize-1][0] - height - 12.0)/3.0;

/* Set the View center and view's drawing ref. for the FRONT view.  */
    center[0] = 0.5*width;
    center[1] = 0.5*height;
    center[2] = 6.0;
    UF_CALL( uc6431( "FRONT_DRF", center, 1.0 ) );
    UF_CALL( uc6443("FRONT_DRF", center, 0.0 ) );

/* Import (add) the FRONT view to the drawing.  Set refpt (the drawing
 * reference point) to position view on drawing.  Check return code.
 */
    refpt[0] = hspace + width/2.0;
    refpt[1] = vspace + 6.0;
    UF_CALL( uc6481(dname,"FRONT_DRF",refpt,1) );

    UF_CALL( UF_VIEW_ask_tag_of_view_name("FRONT_DRF@1",&view_tag) );
    UF_CALL( UF_DRAW_define_view_auto_rect(view_tag) );

/* Set the View center and view's drawing ref. for the RIGHT view.  */
    UF_CALL( uc6431( "RIGHT_DRF", center, 1.0 ) );
    UF_CALL( uc6443("RIGHT_DRF", center, 0.0 ) );

/* Import (add) the RIGHT view to the drawing.  Set refpt (the drawing
 * reference point) to position view on drawing.  Check return code.
 */
    refpt[0] = htwd[dsize-1][1] - hspace - 0.5*height;
    refpt[1] = vspace + 6.0;
    UF_CALL( uc6481(dname,"RIGHT_DRF",refpt,1) );

    UF_CALL( UF_VIEW_ask_tag_of_view_name("RIGHT_DRF@2",&view_tag) );
    UF_CALL( UF_DRAW_define_view_auto_rect(view_tag) );

/* Set the View center and view's drawing ref. for the TOP view.  */
    UF_CALL( uc6431("TOP_DRF",center,1.0) );
    UF_CALL( uc6443("TOP_DRF", center, 0.0 ) );

/* Import (add) the TOP view to the drawing.  Set refpt (the drawing
 * reference point) to position view on drawing.  Check return code.
 */
    refpt[0] = hspace + width/2.0;
    refpt[1] = htwd[dsize-1][0] - vspace - 0.5*height;
    UF_CALL( uc6481(dname,"TOP_DRF",refpt,1) );

/* Change the clipping bounds on the TOP view.  */
/*
    flag = uc6485("TOP_DRF@",borders);
    borders[0] = 0.5 * borders[0];
    borders[1] = 0.5 * borders[1];
    borders[2] = 2.0 * borders[2];
    borders[3] = 2.0 * borders[3];
    flag = uc6486("TOP_DRF@3",borders);
*/
    UF_CALL( UF_VIEW_ask_tag_of_view_name("TOP_DRF@3",&view_tag) );
    UF_CALL( UF_DRAW_define_view_auto_rect(view_tag) );

/* Set the View center and view's drawing ref. for the TFR-ISO view.  */
    UF_CALL( uc6431("TFR-ISO",center,1.0) );
    UF_CALL( uc6443("TFR-ISO", center, 0.0 ) );

/* Import (add) TFR-ISO view to the drawing.  Set refpt (the drawing
 * reference point) to position view on drawing.  Check return code.
 */
    refpt[0] = htwd[dsize-1][1] - hspace - 0.5*height;
    refpt[1] = htwd[dsize-1][0] - vspace - 0.5*height;
    UF_CALL( uc6481(dname,"TFR-ISO",refpt,1) );
    UF_CALL( UF_VIEW_ask_tag_of_view_name("TFR-ISO@4",&view_tag) );
    UF_CALL( UF_DRAW_define_view_auto_rect(view_tag) );

/* Fit the work View at 100% */
    UF_CALL( UF_VIEW_fit_view(NULL_TAG,fraction) );

    return(0);
}
