#include <string.h>
#include <uf_defs.h>
#include <uf.h>
#include <uf_modl.h>
#include <uf_ui.h>
#include "calcproto.h"

/*************************************************************************
*
* calculator
* 
*  This is the lead routine for the calculator creation User Function 
*  program.  This routine calls the subroutines that checks the active
*  part, gets the button arrangement, sets up the assembly, creates the
*  expressions, and creates the geometry and dimensions.  The program
*  also allows edits, creates a drawing, plots the drawing and cleans up.
*
* revisions 
*   rv    date       who    reason for change
*   00    01NOV95    J_P   
*   02   14FEB03     DKF    Changed UF_CALL in calcproto.h to new std.
*                           Removed unreferenced local variables,
*                           Updated error checking
*
*   Note:  This program must not have parts open when it is executed. 
*************************************************************************/

      void ufusr(char *message, int *eflag, int mlen)
{
/******************** Routine variable declaration  *******************/

      char dname[31];     /* drawing name */
 
      int style,          /* calculator style. */
                          /* 1 = vertical, 2 = horizontal */
          flag;           /* user response or flag from functions */
 
      uf_list_p_t holes;  /* List of photocell, lcd, and button slots */
 
      tag_t  top_inst,    /* Tag of top half instance */
             bottom_inst, /* Tag of bottom half instance */
             button_inst; /* Tag of regular button instance */
 
      double height,      /* button area height */
             width;       /* button area width */
 
/******************** Beginning of Executable code *********************/

/* Check out a user function license */
    UF_CALL(UF_initialize());

/* Check for an active part */
    uc1601( "Open the calculator part...", 0 );
    flag = calc_part();
    if(flag) {
        UF_terminate();
        return;
    }

/* Prompt user for the calculator style and overall dimensions. */
    uc1601(" Obtaining overall calculator size", 0 );
    flag = calc_setup( &style, &height, &width );
    if(flag) {
        UF_terminate();
        return;
    }

/* Set up the assembly */
    uc1601(" Setting up the assembly structure", 0 );
	flag = calc_assem( &top_inst, &bottom_inst, &button_inst );
	    if(flag) {
        if( flag == 1 )
            uc1601(" No work part when trying to create assembly",1);
        if( flag == 2 )
            uc1601(" Work part should be metric.  Incorrect units",1);
        UF_terminate();
        return;
    }


/* Create the expressions */
    uc1601(" Creating expressions", 0 );
	flag = calc_expr( style, height, width );
    if(flag) {
        UF_terminate();
        return;
    }


/* Create the geometry */
    uc1601(" Creating model geometry...", 0 );
	UF_CALL( UF_MODL_create_list(&holes) );
    if(flag) {
        UF_terminate();
        return;
    }
    uc1601(" Creating model geometry...", 0 );
    flag = calc_model( top_inst, bottom_inst, button_inst, holes );
    if(flag != 0) {
        UF_CALL( UF_MODL_delete_list(&holes) );
        UF_terminate();
        return;
    }

/* Dimension the model */
    uc1601(" Adding dimensions to the model", 0 );
    flag = calc_dimension( );
    if(flag) {
        UF_CALL( UF_MODL_delete_list(&holes) );
        UF_terminate();
        return;
    }

/* Create a drawing */
    uc1601(" Creating a drawing ...", 0 );
    strcpy(dname, "calcdwg" );
    flag = calc_drawing( dname );
    if(flag) {
        UF_CALL( UF_MODL_delete_list(&holes) );
        UF_terminate();
        return;
    }

/* Free the hole list structure */
    UF_CALL( UF_MODL_delete_list(&holes) );
    if(flag) {
        UF_terminate();
        return;
    }

/* Plot the drawing */
    uc1601(" Creating a plot ...", 0 );

/* Release UG/Open API license */
      UF_terminate();
}


int ufusr_ask_unload(void)
{
    return (UF_UNLOAD_IMMEDIATELY);
}
